@@ -46,9 +46,10 @@
 #include "manifest.h"
 #include "initramfs.h"
 
+
 static void print_version(void);
 static void print_help(const char* name, int is_uninstall, int advanced);
-void free_package(Package *p);
+static void print_module_type(char module_type);
 
 
 
@@ -184,32 +185,32 @@ static int assign_file_type_override(Options *op, char *optarg)
     return TRUE;
 }
 
+static void print_module_type(char module_type)
+{
+    switch (module_type) {
+        case PROPRIETARY:
+            nv_info_msg(NULL, "proprietary");
+            break;
+        case OPEN:
+            nv_info_msg(NULL, "open");
+            break;
+    }
+}
+
 static int print_recommended_module_type_option(Options *op)
 {
-    Package *p;
-    op->logging = 0;
-    op->silent = 1;
+    int num_types;
+    struct module_type_info info;
 
-    /* initialize the user interface; needed for check_nvidia_graphics_devices */
-    if (!ui_init(op)) goto failed;
+    num_types = valid_kernel_module_types(op, &info, TRUE);
 
-    if ((p = parse_manifest(op)) == NULL) goto failed;
-
-    check_for_nvidia_graphics_devices(op, p);
-
-    if (strcmp(p->kernel_module_build_directory, "kernel-open") == 0) {
-        nv_info_msg(NULL, "open");
-    }
-    else {
-        nv_info_msg(NULL, "proprietary");
+    if (num_types <= 0) {
+        ui_error(op, "Unable to determine recommended module type.");
+        return FALSE;
     }
 
-    free_package(p);
+    print_module_type(info.types[info.default_entry]);
     return TRUE;
-
-failed:
-    ui_error(op, "Unable to determine recommended module type.");
-    return FALSE;
 }
 
 /*
@@ -287,7 +288,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             break;
         case 'j':
             if (intval < 1) {
-                nv_error_msg("Invalid concurrency level %d: nvidia-installer "
+                ui_error(op, "Invalid concurrency level %d: nvidia-installer "
                              "will attempt to autodetect the number of CPUs.",
                              intval);
                 intval = 0;
@@ -365,13 +366,13 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->nvidia_modprobe = FALSE; break;
         case FORCE_TLS_OPTION:
             /* This option is no longer used; ignore it. */
-            nv_warning_msg("The '--force-tls' option is deprecated:  "
-                           "nvidia-installer will ignore this option.");
+            ui_warn(op, "The '--force-tls' option is deprecated:  "
+                        "nvidia-installer will ignore this option.");
             break;
         case FORCE_TLS_COMPAT32_OPTION:
             /* This option is no longer used; ignore it. */
-            nv_warning_msg("The '--force-tls-compat32' option is deprecated:  "
-                           "nvidia-installer will ignore this option.");
+            ui_warn(op, "The '--force-tls-compat32' option is deprecated:  "
+                        "nvidia-installer will ignore this option.");
             break;
         case SANITY_OPTION:
             op->sanity = TRUE;
@@ -388,13 +389,13 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             break;
         case NO_RUNLEVEL_CHECK_OPTION:
             /* This option is no longer used; ignore it. */
-            nv_warning_msg("The '--no-runlevel-check' option is deprecated:  "
-                           "nvidia-installer will ignore this option.");
+            ui_warn(op, "The '--no-runlevel-check' option is deprecated:  "
+                        "nvidia-installer will ignore this option.");
             break;
         case 'N':
             /* This option is no longer used; ignore it. */
-            nv_warning_msg("The '--no-network' option is deprecated:  "
-                           "nvidia-installer will ignore this option.");
+            ui_warn(op, "The '--no-network' option is deprecated:  "
+                        "nvidia-installer will ignore this option.");
             break;
         case PRECOMPILED_KERNEL_INTERFACES_PATH_OPTION:
             op->precompiled_kernel_interfaces_path = strval;
@@ -414,7 +415,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             else if (strcasecmp(strval, "no") == 0)
                 op->selinux_option = SELINUX_FORCE_NO;
             else if (strcasecmp(strval, "default")) {
-                nv_error_msg("Invalid parameter for '--force-selinux'");
+                ui_error(op, "Invalid parameter for '--force-selinux'");
                 goto fail;
             }
             break;
@@ -475,7 +476,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             break;
         case INSTALL_VDPAU_WRAPPER_OPTION:
             if (boolval) {
-                nv_error_msg("This driver package does not contain a "
+                ui_error(op, "This driver package does not contain a "
                              "pre-compiled copy of libvdpau.  Please see the "
                              "README for instructions on how to install "
                              "libvdpau.");
@@ -507,12 +508,12 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case GLVND_GLX_CLIENT_OPTION:
             /* This option is no longer used */
             if (!boolval) {
-                nv_error_msg("The --no-glvnd-glx-client option is no longer supported");
+                ui_error(op, "The --no-glvnd-glx-client option is no longer supported");
                 goto fail;
             }
 
-            nv_warning_msg("The '--glvnd-glx-client' option is deprecated:  "
-                           "nvidia-installer will ignore this option.");
+            ui_warn(op, "The '--glvnd-glx-client' option is deprecated:  "
+                        "nvidia-installer will ignore this option.");
             break;
         case GLVND_EGL_CONFIG_FILE_PATH_OPTION:
             op->libglvnd_json_path = strval;
@@ -520,12 +521,12 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case GLVND_EGL_CLIENT_OPTION:
             /* This option is no longer used */
             if (!boolval) {
-                nv_error_msg("The --no-glvnd-egl-client option is no longer supported");
+                ui_error(op, "The --no-glvnd-egl-client option is no longer supported");
                 goto fail;
             }
 
-            nv_warning_msg("The '--glvnd-egl-client' option is deprecated:  "
-                           "nvidia-installer will ignore this option.");
+            ui_warn(op, "The '--glvnd-egl-client' option is deprecated:  "
+                        "nvidia-installer will ignore this option.");
             break;
         case EGL_EXTERNAL_PLATFORM_CONFIG_FILE_PATH_OPTION:
             op->external_platform_json_path = strval;
@@ -552,7 +553,14 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->systemd_sysconf_prefix = strval;
             break;
         case 'm':
-            op->kernel_module_build_directory_override = strval;
+            if (!override_kernel_module_build_directory(op, strval)) {
+                goto fail;
+            }
+            break;
+        case 'M':
+            if (!override_kernel_module_type(op, strval)) {
+                goto fail;
+            }
             break;
         case PRINT_RECOMMENDED_MODULE_TYPE_OPTION:
             print_recommended_kernel_module_after = TRUE;
@@ -567,6 +575,9 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case GBM_BACKEND_DIR_OPTION:
             op->gbm_backend_dir = strval;
             break;
+        case GBM_BACKEND_DIR32_OPTION:
+            op->compat32_gbm_backend_dir = strval;
+            break;
         default:
             goto fail;
         }
@@ -622,7 +633,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
     return;
     
  fail:
-    nv_error_msg("Invalid commandline, please run `%s --help` "
+    ui_error(op, "Invalid commandline, please run `%s --help` "
                  "for usage information.", argv[0]);
     nvfree((void*)op);
     exit(1);
@@ -649,7 +660,10 @@ int main(int argc, char *argv[])
         fprintf(stderr, "\nOut of memory error.\n\n");
         return 1;
     }
-    
+
+    /* check for supported PCI devices */
+    pci_device_scan(op);
+
     /* parse the commandline options */
     
     parse_commandline(argc, argv, op);
